@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2022-2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -418,16 +418,15 @@ setup(const TestData& tdata)
 
 	/* create maildirs */
 	for (size_t i = 0; i != tdata.num_maildirs; ++i) {
-		const auto mdir = format("%s/maildir-%zu", top_maildir.c_str(), i);
+		const auto mdir = mu_format("{}/maildir-{}", top_maildir, i);
 		auto res = maildir_mkdir(mdir);
 		g_assert(!!res);
 	}
 	const auto rx = Regex::make("@ID@");
 	/* create messages */
 	for (size_t n = 0; n != tdata.num_messages; ++n) {
-		auto mpath = format("%s/maildir-%zu/cur/msg-%zu:2,S",
-				    top_maildir.c_str(),
-				    n % tdata.num_maildirs,
+		auto mpath = mu_format("{}/maildir-{}/cur/msg-{}:2,S",
+				    top_maildir, n % tdata.num_maildirs,
 				    n);
 		std::ofstream stream(mpath);
 		auto msg = message(*rx, n);
@@ -441,9 +440,9 @@ tear_down()
 {
 	/* ugly */
 	GError *err{};
-	const auto cmd{format("/bin/rm -rf '%s' '%s'", BENCH_MAILDIRS, BENCH_STORE)};
+	const auto cmd{mu_format("/bin/rm -rf '{}' '{}'", BENCH_MAILDIRS, BENCH_STORE)};
 	if (!g_spawn_command_line_sync(cmd.c_str(), NULL, NULL, NULL, &err)) {
-		g_warning("error: %s\n", err ? err->message : "?");
+		mu_warning("error: {}", err ? err->message : "?");
 		g_clear_error(&err);
 	}
 }
@@ -465,7 +464,7 @@ benchmark_indexer(gconstpointer testdata)
 	auto start = Clock::now();
 
 	{
-		auto store{Store::make_new(BENCH_STORE, BENCH_MAILDIRS, {}, {})};
+		auto store{Store::make_new(BENCH_STORE, BENCH_MAILDIRS)};
 		g_assert_true(!!store);
 		Indexer::Config conf{};
 		conf.max_threads = tdata->num_threads;
@@ -494,7 +493,9 @@ int
 main(int argc, char *argv[])
 {
 	size_t num_maildirs{}, num_messages{};
-	g_test_init(&argc, &argv, nullptr);
+
+	mu_test_init(&argc, &argv);
+
 	if (g_test_perf()) {
 		num_maildirs = 20;
 		num_messages = 5000;
