@@ -44,7 +44,8 @@
 #include <glib/gprintf.h>
 
 #include "mu-utils.hh"
-#include "mu-utils-format.hh"
+#include "mu-unbroken.hh"
+
 #include "mu-error.hh"
 #include "mu-option.hh"
 
@@ -113,12 +114,28 @@ gx_utf8_flatten(const gchar* str, gssize len)
 
 } // namespace
 
+bool
+Mu::contains_unbroken_script(const char *str)
+{
+	while (str && *str) {
+		auto uc = g_utf8_get_char(str);
+		if (is_unbroken_script(uc))
+			return true;
+		str = g_utf8_next_char(str);
+	}
+
+	return false;
+}
+
 std::string // gx_utf8_flatten
 Mu::utf8_flatten(const char* str)
 {
 	if (!str)
 		return {};
 
+	if (contains_unbroken_script(str))
+		return std::string{str};
+
 	// the pure-ascii case
 	if (g_str_is_ascii(str)) {
 		auto    l = g_ascii_strdown(str, -1);
@@ -190,6 +207,60 @@ Mu::utf8_clean(const std::string& dirty)
 	return std::string{g_strstrip(gstr->str)};
 }
 
+
+std::string
+Mu::utf8_wordbreak(const std::string& txt)
+{
+	g_autoptr(GString) gstr = g_string_sized_new(txt.length());
+
+	bool spc{};
+	for (auto cur = txt.c_str(); cur && *cur; cur = g_utf8_next_char(cur)) {
+		const gunichar uc = g_utf8_get_char(cur);
+
+		if (g_unichar_iscntrl(uc)) {
+			g_string_append_c(gstr, ' ');
+			continue;
+		}
+		// inspired by Xapian's termgenerator.
+
+		switch(uc) {
+		case '\'':
+		case '&':
+		case 0xb7:
+		case 0x5f4:
+		case 0x2019:
+		case 0x201b:
+		case 0x2027:
+		case ',':
+		case '.':
+		case ';':
+		case '+':
+		case '#':
+		case '-':
+		case 0x037e: // GREEK QUESTION MARK
+		case 0x0589: // ARMENIAN FULL STOP
+		case 0x060D: // ARABIC DATE SEPARATOR
+		case 0x07F8: // NKO COMMA
+		case 0x2044: // FRACTION SLASH
+		case 0xFE10: // PRESENTATION FORM FOR VERTICAL COMMA
+		case 0xFE13: // PRESENTATION FORM FOR VERTICAL COLON
+		case 0xFE14: // PRESENTATION FORM FOR VERTICAL SEMICOLON
+			if (spc)
+				break;
+			spc = true;
+			g_string_append_c(gstr, ' ');
+			break;
+		default:
+			spc = false;
+			g_string_append_unichar(gstr, uc);
+			break;
+		}
+	}
+
+	return std::string{g_strstrip(gstr->str)};
+}
+
+
 std::string
 Mu::remove_ctrl(const std::string& str)
 {
@@ -306,62 +377,7 @@ Mu::quote(const std::string& str)
 	return res + "\"";
 }
 
-std::string
-Mu::format(const char* frm, ...)
-{
-	va_list args;
-
-	va_start(args, frm);
-	auto str = vformat(frm, args);
-	va_end(args);
-
-	return str;
-}
-
-std::string
-Mu::vformat(const char* frm, va_list args)
-{
-	char*      s{};
-	const auto res = g_vasprintf(&s, frm, args);
-	if (res == -1) {
-		std::cerr << "string format failed" << std::endl;
-		return {};
-	}
-
-	std::string str{s};
-	g_free(s);
-
-	return str;
-}
-
-std::string
-Mu::time_to_string(const char *frm, time_t t, bool utc)
-{
-	g_return_val_if_fail(frm, "");
-
-	GDateTime* dt = std::invoke([&] {
-		if (utc)
-			return g_date_time_new_from_unix_utc(t);
-		else
-			return g_date_time_new_from_unix_local(t);
-	});
-
-	if (!dt) {
-		g_warning("time_t out of range: <%" G_GUINT64_FORMAT ">",
-			  static_cast<guint64>(t));
-		return {};
-	}
-
-	frm = frm ? frm : "%c";
-	auto datestr{to_string_opt_gchar(g_date_time_format(dt, frm))};
-	g_date_time_unref(dt);
-	if (!datestr)
-		g_warning("failed to format time with format '%s'", frm);
-
-	return datestr.value_or("");
-}
-
-static Option<int64_t>
+static Option<::time_t>
 delta_ymwdhMs(const std::string& expr)
 {
 	char* endptr;
@@ -391,7 +407,7 @@ delta_ymwdhMs(const std::string& expr)
 		then =
 		    g_date_time_add_full(now, -years, -months, -days, -hours, -minutes, -seconds);
 
-	auto  t = std::max<int64_t>(0, g_date_time_to_unix(then));
+	auto  t = std::max<::time_t>(0, g_date_time_to_unix(then));
 
 	g_date_time_unref(then);
 	g_date_time_unref(now);
@@ -399,7 +415,7 @@ delta_ymwdhMs(const std::string& expr)
 	return t;
 }
 
-static Option<int64_t>
+static Option<::time_t>
 special_date_time(const std::string& d, bool is_first)
 {
 	if (d == "now")
@@ -468,12 +484,12 @@ fixup_month(struct tm* tbuf)
  }
 
 
-Option<int64_t>
-Mu::parse_date_time(const std::string& dstr, bool is_first)
+Option<::time_t>
+Mu::parse_date_time(const std::string& dstr, bool is_first, bool utc)
 {
 	struct tm tbuf{};
 	GDateTime *dtime{};
-	int64_t t;
+	gint64 t;
 
 	/* one-sided dates */
 	if (dstr.empty())
@@ -498,16 +514,24 @@ Mu::parse_date_time(const std::string& dstr, bool is_first)
 		return Nothing;
 
 	fixup_month(&tbuf);
-	dtime = g_date_time_new_local(tbuf.tm_year + 1900,
+	dtime = utc ?
+		g_date_time_new_utc(tbuf.tm_year + 1900,
+				      tbuf.tm_mon + 1,
+				      tbuf.tm_mday,
+				      tbuf.tm_hour,
+				      tbuf.tm_min,
+				      tbuf.tm_sec) :
+		g_date_time_new_local(tbuf.tm_year + 1900,
 				      tbuf.tm_mon + 1,
 				      tbuf.tm_mday,
 				      tbuf.tm_hour,
 				      tbuf.tm_min,
 				      tbuf.tm_sec);
+
 	t = g_date_time_to_unix(dtime);
 	g_date_time_unref(dtime);
 
-	return std::max<int64_t>(t, 0);
+	return to_time_t(t);
 }
 
 
@@ -687,32 +711,3 @@ Mu::fputs_encoded (const std::string& str, FILE *stream)
 
 	return (rv == EOF) ? false: true;
 }
-
-__attribute__((format(printf, 2, 0)))
-static bool
-print_args (FILE *stream, const char *frm, va_list args)
-{
-	gchar *str;
-	gboolean rv;
-
-	str = g_strdup_vprintf (frm, args);
-	rv = fputs_encoded (str, stream);
-	g_free (str);
-
-	return rv;
-}
-
-bool
-Mu::print_encoded (const char *frm, ...)
-{
-	va_list args;
-	gboolean rv;
-
-	g_return_val_if_fail (frm, false);
-
-	va_start (args, frm);
-	rv = print_args (stdout, frm, args);
-	va_end (args);
-
-	return rv;
-}
