@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022-2023 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2022-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -58,6 +58,53 @@ using namespace Mu;
  * helpers
  */
 
+
+
+/**
+ * array of associated pair elements -- like an alist
+ * but based on std::array and thus can be constexpr
+ */
+template<typename T1, typename T2, std::size_t N>
+      using AssocPairs = std::array<std::pair<T1, T2>, N>;
+
+
+/**
+  * Get the first value of the pair where the second element is @param s.
+  *
+  * @param p AssocPairs
+  * @param s some second pair value
+  *
+  * @return the matching first pair value, or Nothing if not found.
+  */
+template<typename P>
+constexpr Option<typename P::value_type::first_type>
+to_first(const P& p, typename P::value_type::second_type s)
+{
+	for (const auto& item: p)
+		if (item.second == s)
+			return item.first;
+	return Nothing;
+}
+
+/**
+  * Get the second value of the pair where the first element is @param f.
+  *
+  * @param p AssocPairs
+  * @param f some first pair value
+  *
+  * @return the matching second pair value, or Nothing if not found.
+  */
+template<typename P>
+constexpr Option<typename P::value_type::second_type>
+to_second(const P& p, typename P::value_type::first_type f)
+{
+	for (const auto& item: p)
+		if (item.first == f)
+			return item.second;
+	return Nothing;
+}
+
+
 /**
  * Options-specific array-bases type that maps some enum to a <name, description> pair
  */
@@ -137,17 +184,19 @@ options_map(const IE& ie)
 	return map;
 }
 
-
-
 // transformers
 
-
 // Expand the path using wordexp
 static const std::function ExpandPath = [](std::string filepath)->std::string {
 	if (auto&& res{expand_path(filepath)}; !res)
 		throw CLI::ValidationError{res.error().what()};
 	else
-		return filepath = std::move(res.value());
+		return res.value();
+};
+
+// Canonicalize path
+static const std::function CanonicalizePath = [](std::string filepath)->std::string {
+	return filepath = canonicalize_filename(filepath);
 };
 
 /*
@@ -236,18 +285,30 @@ sub_extract(CLI::App& sub, Options& opts)
 	sub.add_option("--target-dir", opts.extract.targetdir,
 		       "Target directory for saving")
 		->type_name("<dir>")
-		->default_str("<current>")->default_val(".")
-		->transform(ExpandPath, "expand path");
-	sub.add_option("message", opts.extract.message,
-		       "Path to message file")->required()
-		->type_name("<message-path>");
+		->transform(ExpandPath, "expand target path")
+		->default_str("<current>")
+		->default_val(".");
 	sub.add_flag("--uncooked,-u", opts.extract.uncooked,
 		       "Avoid massaging extracted file-names");
+	// optional; otherwise use standard-input
+	sub.add_option("message-path", opts.extract.message,
+		       "Path to message file")
+		->type_name("<message-path>");
+
+	sub.add_option("--matches", opts.extract.filename_rx,
+		       "Regular expression for files to save")
+		->type_name("<filename-rx>")
+		->excludes("--parts")
+		->excludes("--save-attachments")
+		->excludes("--save-all");
+
+	// backward compat: filename-rx as non-option
 	sub.add_option("filename-rx", opts.extract.filename_rx,
 		       "Regular expression for files to save")
 		->type_name("<filename-rx>")
 		->excludes("--parts")
 		->excludes("--save-attachments")
+		->excludes("--matches")
 		->excludes("--save-all");
 }
 
@@ -278,12 +339,6 @@ sub_find(CLI::App& sub, Options& opts)
 			{ Format::Json,
 			  {"json", "JSON"}
 			},
-			{ Format::XQuery,
-			  {"xquery", "Show Xapian query (for debugging)"}
-			},
-			{ Format::MQuery,
-			  {"mquery", "Show mu query for (for debugging)"}
-			},
 		}};
 
 	sub.add_flag("--threads,-t", opts.find.threads,
@@ -292,6 +347,8 @@ sub_find(CLI::App& sub, Options& opts)
 		     "Show only one of messages with same message-id");
 	sub.add_flag("--include-related,-r", opts.find.include_related,
 		     "Include related messages in results");
+	sub.add_flag("--analyze,-a", opts.find.analyze,
+		     "Analyze the query");
 
 	const auto fhelp = options_help(FormatInfos, Format::Plain);
 	const auto fmap = options_map(FormatInfos);
@@ -320,7 +377,7 @@ sub_find(CLI::App& sub, Options& opts)
 			smap.emplace(std::string(1, field.shortcut), field.id);
 			if (!sopts.empty())
 				sopts += ", ";
-			sopts += format("%.*s|%c", STR_V(field.name), field.shortcut);
+			sopts += mu_format("{}|{}", field.name, field.shortcut);
 		}
 	});
 	sub.add_option("--sortfield,-s", opts.find.sortfield,
@@ -330,7 +387,8 @@ sub_find(CLI::App& sub, Options& opts)
 		->default_val(Field::Id::Date)
 		->transform(CLI::CheckedTransformer(smap));
 
-	sub.add_flag("--reverse,-z", opts.find.reverse, "Sort in descending order");
+	sub.add_flag("--reverse,-z", opts.find.reverse,
+		     "Sort in descending order");
 
 	sub.add_option("--bookmark,-b", opts.find.bookmark,
 		       "Use bookmarked query")
@@ -341,7 +399,7 @@ sub_find(CLI::App& sub, Options& opts)
 	sub.add_option("--linksdir", opts.find.linksdir,
 		       "Use bookmarked query")
 		->type_name("<dir>")
-		->transform(ExpandPath, "expand path");
+		->transform(ExpandPath, "expand linksdir path");
 
 	sub.add_option("--summary-len", opts.find.summary_len,
 		       "Use up to so many lines for the summary")
@@ -352,7 +410,8 @@ sub_find(CLI::App& sub, Options& opts)
 		       "Command to execute on message file")
 		->type_name("<command>");
 
-	sub.add_option("query", opts.find.query, "Search query pattern(s)")
+	sub.add_option("query", opts.find.query,
+		       "Search query pattern(s)")
 		->type_name("<query>");
 }
 
@@ -364,7 +423,6 @@ sub_help(CLI::App& sub, Options& opts)
 		->type_name("<command>");
 }
 
-
 static void
 sub_index(CLI::App& sub, Options& opts)
 {
@@ -372,35 +430,57 @@ sub_index(CLI::App& sub, Options& opts)
 		       "Skip based on dir-timestamps");
 	sub.add_flag("--nocleanup", opts.index.nocleanup,
 		       "Don't clean up database after indexing");
+	sub.add_flag("--reindex", opts.index.reindex,
+		     "Perform a complete reindexing");
 }
 
 
 static void
 sub_info(CLI::App& sub, Options& opts)
 {
-	// nothing to do.
+	sub.add_option("topic", opts.info.topic,
+		       "Information topic")
+		->type_name("<topic>") ;
 }
 
 static void
 sub_init(CLI::App& sub, Options& opts)
 {
-	sub.add_option("--maildir,-m", opts.init.maildir,
-		       "Top of the maildir")
+	const auto default_mdir = std::invoke([]()->std::string {
+		if (const auto mdir_env{::getenv("MAILDIR")}; mdir_env)
+			return mdir_env;
+		else if (const auto mdir_home = ::join_paths(g_get_home_dir(), "Maildir");
+			 check_dir(mdir_home))
+			return mdir_home;
+		else
+			return {};
+	});
+
+	sub.add_option("--maildir,-m", opts.init.maildir, "Top of the maildir")
 		->type_name("<maildir>")
-		->transform(ExpandPath, "expand path");
+		->default_val(default_mdir)
+		->transform(ExpandPath, "expand maildir path");
 	sub.add_option("--my-address", opts.init.my_addresses,
-		       "Personal e-mail addresses")
+		       "Personal e-mail address or regexp")
 		->type_name("<address>");
+	sub.add_option("--ignored-address", opts.init.ignored_addresses,
+		       "Ignored e-mail address or regexp")
+		->type_name("<address>");
+
 	sub.add_option("--max-message-size", opts.init.max_msg_size,
 		       "Maximum allowed message size in bytes");
 	sub.add_option("--batch-size", opts.init.batch_size,
 		       "Maximum size of database transaction");
+	sub.add_option("--support-ngrams", opts.init.support_ngrams,
+		       "Support CJK n-grams if for querying/indexing");
 	sub.add_flag("--reinit", opts.init.reinit,
 		       "Re-initialize database with current settings")
 		->excludes("--maildir")
 		->excludes("--my-address")
+		->excludes("--ignored-address")
 		->excludes("--max-message-size")
-		->excludes("--batch-size");
+		->excludes("--batch-size")
+		->excludes("--support-ngrams");
 }
 
 static void
@@ -415,6 +495,31 @@ sub_mkdir(CLI::App& sub, Options& opts)
 		->required();
 }
 
+
+static void
+sub_move(CLI::App& sub, Options& opts)
+{
+	sub.add_flag("--change-name", opts.move.change_name,
+		     "Change name of target file");
+	sub.add_flag("--update-dups", opts.move.update_dups,
+		     "Update duplicate messages too");
+	sub.add_flag("--dry-run,-n", opts.move.dry_run,
+		     "Print target name, but do not change anything");
+
+	sub.add_option("--flags", opts.move.flags, "Target flags")
+		->type_name("<flags>");
+
+	sub.add_option("source", opts.move.src, "Message file to move")
+		->type_name("<message-path>")
+		->transform(ExpandPath, "expand source path")
+		->transform(CanonicalizePath, "canonicalize source path")
+		->required();
+	sub.add_option("destination", opts.move.dest,
+		       "Destination maildir")
+		->type_name("<maildir>");
+}
+
+
 static void
 sub_remove(CLI::App& sub, Options& opts)
 {
@@ -431,6 +536,10 @@ sub_server(CLI::App& sub, Options& opts)
 	sub.add_option("--eval", opts.server.eval,
 		       "Evaluate mu server expression")
 		->excludes("--commands");
+	sub.add_flag("--allow-temp-file", opts.server.allow_temp_file,
+		     "Allow for the temp-file optimization")
+		->excludes("--commands");
+
 }
 
 static void
@@ -438,20 +547,23 @@ sub_verify(CLI::App& sub, Options& opts)
 {
 	sub_crypto(sub, opts.verify);
 
-	sub.add_option("files", opts.verify.files,
+	// optional; otherwise use standard-input
+	sub.add_option("message-paths", opts.verify.files,
 		       "Message files to verify")
-		->type_name("<message-file>")
-		->required();
+		->type_name("<message-path>");
 }
 
 static void
 sub_view(CLI::App& sub, Options& opts)
 {
 	using Format = Options::View::Format;
-	static constexpr InfoEnum<Format, 2> FormatInfos = {{
+	static constexpr InfoEnum<Format, 3> FormatInfos = {{
 			{ Format::Plain,
 			  {"plain", "Plain output"}
 			},
+			{ Format::Html,
+			  {"html", "Plain output with HTML body"}
+			},
 			{ Format::Sexp,
 			  {"sexp", "S-expressions"}
 			},
@@ -477,10 +589,10 @@ sub_view(CLI::App& sub, Options& opts)
 	sub.add_flag("--terminate", opts.view.terminate,
 		     "Insert form-feed after each message");
 
-	sub.add_option("files", opts.view.files,
+	// optional; otherwise use standard-input
+	sub.add_option("message-paths", opts.view.files,
 		       "Message files to view")
-		->type_name("<file>")
-		->required();
+		->type_name("<message-path>");
 }
 
 
@@ -513,7 +625,7 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 		{ SubCommand::Fields,
 		  {Category::None,
-		  "fields", "Show a information about search fields", sub_fields}
+		  "fields", "Superseded by 'mu info'", sub_fields}
 		},
 		{ SubCommand::Find,
 		  {Category::NeedsReadOnlyStore,
@@ -529,7 +641,7 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 		{ SubCommand::Info,
 		  {Category::NeedsReadOnlyStore,
-		  "info", "Show information about the message store database", sub_info }
+		  "info", "Show information", sub_info }
 		},
 		{ SubCommand::Init,
 		  {Category::NeedsWritableStore,
@@ -539,6 +651,10 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		  {Category::None,
 		  "mkdir", "Create a new Maildir", sub_mkdir }
 		},
+		{ SubCommand::Move,
+		  {Category::NeedsWritableStore,
+		   "move", "Move a message or change flags", sub_move }
+		},
 		{ SubCommand::Remove,
 		  {Category::NeedsWritableStore,
 		  "remove", "Remove message from file-system and database", sub_remove }
@@ -609,7 +725,7 @@ static Result<Options>
 cmd_help(const CLI::App& app, Options& opts)
 {
 	if (opts.help.command.empty()) {
-		std::cout << app.help() << "\n";
+		mu_println("{}", app.help());
 		return Ok(std::move(opts));
 	}
 
@@ -623,7 +739,7 @@ cmd_help(const CLI::App& app, Options& opts)
 			return show_manpage(opts, "mu-" + opts.help.command);
 
 	return Err(Error::Code::Command,
-		   "no help available for '%s'", opts.help.command.c_str());
+		   "no help available for '{}'", opts.help.command);
 }
 
 bool
@@ -645,7 +761,8 @@ add_global_options(CLI::App& cli, Options& opts)
 
 	cli.add_flag("-q,--quiet", opts.quiet, "Hide non-essential output");
 	cli.add_flag("-v,--verbose", opts.verbose, "Show verbose output");
-	cli.add_flag("--log-stderr", opts.log_stderr, "Log to stderr");
+	cli.add_flag("--log-stderr", opts.log_stderr, "Log to stderr")
+		->group(""/*always hide*/);
 	cli.add_flag("--nocolor", opts.nocolor, "Don't show ANSI colors")
 		->default_val(Options::default_no_color())
 		->default_str(Options::default_no_color() ? "<true>" : "<false>");
@@ -660,18 +777,18 @@ Options::make(int argc, char *argv[])
 	CLI::App app{"mu mail indexer/searcher", "mu"};
 
 	app.description(R"(mu mail indexer/searcher
-Copyright (C) 2008-2022 Dirk-Jan C. Binnema
+Copyright (C) 2008-2023 Dirk-Jan C. Binnema
 
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
 )");
-	 app.set_version_flag("-V,--version", PACKAGE_VERSION);
-	 app.set_help_flag("-h,--help", "Show help informmation");
-	 app.set_help_all_flag("--help-all");
-	 app.require_subcommand(0, 1);
+	app.set_version_flag("-V,--version", PACKAGE_VERSION);
+	app.set_help_flag("-h,--help", "Show help informmation");
+	app.set_help_all_flag("--help-all");
+	app.require_subcommand(0, 1);
 
-	 add_global_options(app, opts);
+	add_global_options(app, opts);
 
 	/*
 	 * subcommands
@@ -679,7 +796,7 @@ There is NO WARRANTY, to the extent permitted by law.
 	 * we keep around a map of the subcommand pointers, so we can
 	 * easily find the chosen one (if any) later.
 	 */
-	 for (auto&& cmdinfo: SubCommandInfos) {
+	for (auto&& cmdinfo: SubCommandInfos) {
 		//const auto cmdtype = cmdinfo.first;
 		const auto name{std::string{cmdinfo.second.name}};
 		const auto help{std::string{cmdinfo.second.help}};
@@ -704,13 +821,13 @@ There is NO WARRANTY, to the extent permitted by law.
 					opts.muhome, "Specify alternative mu directory")
 				->envname("MUHOME")
 				->type_name("<dir>")
-				->transform(ExpandPath, "expand path");
-	 }
+				->transform(ExpandPath, "expand muhome path");
+	}
 
-	 /* add scripts (if supported) as semi-subscommands as well */
-	 const auto scripts = add_scripts(app, opts);
+	/* add scripts (if supported) as semi-subcommands as well */
+	const auto scripts = add_scripts(app, opts);
 
-	 try {
+	try {
 		app.parse(argc, argv);
 
 		// find the chosen sub command,  if any.
@@ -738,13 +855,13 @@ There is NO WARRANTY, to the extent permitted by law.
 			return cmd_help(app, opts);
 
 	} catch (const CLI::CallForHelp& cfh) {
-		std::cout << app.help() << std::flush;
+		mu_println("{}", app.help());
 	} catch (const CLI::CallForAllHelp& cfah) {
-		std::cout << app.help("", CLI::AppFormatMode::All) << std::flush;
+		mu_println("{}", app.help("", CLI::AppFormatMode::All));
 	} catch (const CLI::CallForVersion&) {
-		std::cout << "version " << PACKAGE_VERSION << "\n";
+		mu_println("version {}", PACKAGE_VERSION);
 	} catch (const CLI::ParseError& pe) {
-		return Err(Error::Code::InvalidArgument, "%s", pe.what());
+		return Err(Error::Code::InvalidArgument, "{}", pe.what());
 	}  catch (...) {
 		return Err(Error::Code::Internal, "error parsing arguments");
 	}
@@ -769,6 +886,11 @@ Options::category(Options::SubCommand sub)
 static constexpr bool
 validate_subcommand_ids()
 {
+	size_t val{};
+	for (auto& cmd: Options::SubCommands)
+		if (static_cast<size_t>(cmd) != val++)
+			return false;
+
 	for (auto u = 0U; u != SubCommandInfos.size(); ++u)
 		if (static_cast<size_t>(SubCommandInfos.at(u).first) != u)
 			return false;
@@ -793,12 +915,41 @@ test_ids()
 
 #ifdef BUILD_TESTS
 
+enum struct TestEnum { A, B, C };
+constexpr AssocPairs<TestEnum, std::string_view, 3>
+test_epairs = {{
+	{TestEnum::A, "a"},
+	{TestEnum::B, "b"},
+	{TestEnum::C, "c"},
+}};
+
+static constexpr Option<std::string_view>
+to_name(TestEnum te)
+{
+	return to_second(test_epairs, te);
+}
+
+static constexpr Option<TestEnum>
+to_type(std::string_view name)
+{
+	return to_first(test_epairs, name);
+
+}
+
+static void
+test_enum_pairs(void)
+{
+	assert_equal(to_name(TestEnum::A).value(), "a");
+	g_assert_true(to_type("c").value() ==  TestEnum::C);
+}
+
 int
 main(int argc, char* argv[])
 {
 	mu_test_init(&argc, &argv);
 
 	g_test_add_func("/options/ids", test_ids);
+	g_test_add_func("/option/enum-pairs", test_enum_pairs);
 
 	return g_test_run();
 }
